@@ -2,7 +2,8 @@
  * dpkg-deb - construction and deconstruction of *.deb archives
  * info.c - providing information
  *
- * Copyright (C) 1994,1995 Ian Jackson <iwj10@cus.cam.ac.uk>
+ * Copyright (C) 1994,1995 Ian Jackson <ian@chiark.greenend.org.uk>
+ * Copyright (C) 2001 Wichert Akkerman
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as
@@ -18,6 +19,7 @@
  * License along with dpkg; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
+#include <config.h>
 
 #include <stdio.h>
 #include <string.h>
@@ -32,8 +34,8 @@
 #include <limits.h>
 #include <ctype.h>
 
-#include <config.h>
 #include <dpkg.h>
+#include <dpkg-db.h>
 #include <myopt.h>
 #include "dpkg-deb.h"
 
@@ -64,9 +66,9 @@ static void info_prepare(const char *const **argvp,
   
   *debarp= *(*argvp)++;
   if (!*debarp) badusage(_("--%s needs a .deb filename argument"),cipaction->olong);
-  /* This created a temporary directory, so ignore the warning. */
+  /* This creates a temporary directory, so ignore the warning. */
   if ((dbuf= tempnam(NULL,"dpkg")) == NULL)
-    ohshite(_("failed to make temporary filename"));
+    ohshite(_("failed to make temporary directoryname"));
   *directoryp= dbuf;
 
   if (!(c1= m_fork())) {
@@ -94,13 +96,16 @@ static void info_spew(const char *debar, const char *directory,
     } else if (errno == ENOENT) {
       if (fprintf(stderr, _("dpkg-deb: `%.255s' contains no control component `%.255s'\n"),
                   debar, component) == EOF) werr("stderr");
-      re= 1;
+      re++;
     } else {
       ohshite(_("open component `%.255s' (in %.255s) failed in an unexpected way"),
               component, directory);
     }
   }
-  if (re) ohshit(_("at least one requested control component missing"));
+  if (re==1)
+    ohshit(_("One requested control component is missing"));
+  else if (re>1)
+    ohshit(_("%d requested control components are missing"), re);
 }
 
 static void info_list(const char *debar, const char *directory) {
@@ -205,6 +210,21 @@ static void info_field(const char *debar, const char *directory,
   if (ferror(stdout)) werr("stdout");
 }
 
+void do_showinfo(const char* const* argv) {
+  const char *debar, *directory;
+  struct pkginfo *pkg;
+  struct lstitem* fmt = parseformat(showformat);
+
+  if (!fmt)
+    ohshit(_("Error in format"));
+
+  info_prepare(&argv,&debar,&directory,1);
+
+  parsedb(CONTROLFILE, pdb_ignorefiles, &pkg, NULL, NULL);
+  show1package(fmt,pkg);
+}
+
+
 void do_info(const char *const *argv) {
   const char *debar, *directory;
 
@@ -235,3 +255,5 @@ void do_contents(const char *const *argv) {
   if (!(debar= *argv++) || *argv) badusage(_("--contents takes exactly one argument"));
   extracthalf(debar, 0, "tv", 0);
 }
+/* vi: sw=2
+ */
