@@ -18,6 +18,7 @@
  * License along with dpkg; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
+#include <config.h>
 
 #include <stdio.h>
 #include <string.h>
@@ -30,7 +31,6 @@
 #include <unistd.h>
 #include <fcntl.h>
 
-#include <config.h>
 #include <dpkg.h>
 #include <dpkg-db.h>
 #include "parsedump.h"
@@ -142,7 +142,7 @@ int parsedb(const char *filename, enum parsedbflags flags,
       while (!EOF_mmap(dataptr, endptr) && !isspace(c) && c!=':' && c!=MSDOS_EOF_CHAR)
         c= getc_mmap(dataptr);
       fieldlen= dataptr - fieldstart - 1;
-      while (EOF_mmap(dataptr, endptr) && c != '\n' && isspace(c)) c= getc_mmap(dataptr);
+      while (!EOF_mmap(dataptr, endptr) && c != '\n' && isspace(c)) c= getc_mmap(dataptr);
       if (EOF_mmap(dataptr, endptr))
         parseerr(NULL,filename,lno, warnto,warncount,&newpig,0,
                  _("EOF after field name `%.*s'"),fieldlen,fieldstart);
@@ -199,7 +199,7 @@ int parsedb(const char *filename, enum parsedbflags flags,
            fip++, ip++);
       if (fip->name) {
 	value= realloc(value,valuelen+1);
-	strncpy(value,valuestart,valuelen);
+	memcpy(value,valuestart,valuelen);
 	*(value+valuelen)= 0;
         if (*ip++)
           parseerr(NULL,filename,lno, warnto,warncount,&newpig,0,
@@ -241,8 +241,6 @@ int parsedb(const char *filename, enum parsedbflags flags,
     if (flags & pdb_recordavailable)
       parsemustfield(NULL,filename,lno, warnto,warncount,&newpig,1,
                      (const char **)&newpifp->architecture, "architecture");
-    else if (newpifp->architecture && *newpifp->architecture)
-      newpifp->architecture= NULL;
 
     /* Check the Config-Version information:
      * If there is a Config-Version it is definitely to be used, but
