@@ -1,3 +1,6 @@
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
 #include <tarfn.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -76,7 +79,11 @@ static int
 SetModes(TarInfo * i)
 {
 	struct utimbuf t;
+#ifdef HAVE_LCHOWN
+	lchown(i->Name, i->UserID, i->GroupID);
+#else
 	chown(i->Name, i->UserID, i->GroupID);
+#endif
 	chmod(i->Name, i->Mode & ~S_IFMT);
 	t.actime = time(0);
 	t.modtime = i->ModTime;
@@ -146,7 +153,7 @@ static const TarFunctions	functions = {
 int
 main(int argc, char * * argv)
 {
-	int	status = TarExtractor((void *)0, &functions);
+	int	status = TarExtractor(NULL, &functions);
 
 	if ( status == -1 ) {
 		fflush(stdout);
