@@ -19,6 +19,7 @@
  * License along with dpkg; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
+#include <config.h>
 
 #include <errno.h>
 #include <stdio.h>
@@ -37,7 +38,6 @@
 #define obstack_chunk_alloc m_malloc
 #define obstack_chunk_free free
 
-#include <config.h>
 #include <dpkg.h>
 #include <dpkg-db.h>
 #include <tarfn.h>
@@ -525,10 +525,16 @@ int tarobject(struct TarInfo *ti) {
     debug(dbg_eachfiledetail,"tarobject FIFO");
     newtarobject_allmodes(fnamenewvb.buf,ti, nifd->namenode->statoverride);
     break;
-  case CharacterDevice: case BlockDevice:
-    if (mknod(fnamenewvb.buf,0,ti->Device))
+  case CharacterDevice:
+    if (mknod(fnamenewvb.buf,S_IFCHR, ti->Device))
       ohshite(_("error creating device `%.255s'"),ti->Name);
-    debug(dbg_eachfiledetail,"tarobject CharacterDevice|BlockDevice");
+    debug(dbg_eachfiledetail,"tarobject CharacterDevice");
+    newtarobject_allmodes(fnamenewvb.buf,ti, nifd->namenode->statoverride);
+    break; 
+  case BlockDevice:
+    if (mknod(fnamenewvb.buf,S_IFBLK, ti->Device))
+      ohshite(_("error creating device `%.255s'"),ti->Name);
+    debug(dbg_eachfiledetail,"tarobject BlockDevice");
     newtarobject_allmodes(fnamenewvb.buf,ti, nifd->namenode->statoverride);
     break; 
   case HardLink:
