@@ -16,7 +16,10 @@ $parsechangelog= 'dpkg-parsechangelog';
 grep($capit{lc $_}=$_, qw(Pre-Depends Standards-Version Installed-Size
 			  Build-Depends Build-Depends-Indep 
 			  Build-Conflicts Build-Conflicts-Indep));
-
+@pkg_dep_fields = qw(Replaces Provides Depends Pre-Depends Recommends Suggests
+                     Conflicts Enhances);
+@src_dep_fields = qw(Build-Depends Build-Depends-Indep
+                     Build-Conflicts Build-Conflicts-Indep);
 
 $substvar{'Format'}= 1.7;
 $substvar{'Newline'}= "\n";
@@ -102,21 +105,7 @@ sub substvars {
 sub outputclose {
     my ($dosubstvars) = @_;
     for $f (keys %f) { $substvar{"F:$f"}= $f{$f}; }
-    if (length($varlistfile) and $dosubstvars) {
-        $varlistfile="./$varlistfile" if $varlistfile =~ m/\s/;
-        if (open(SV,"< $varlistfile")) {
-            while (<SV>) {
-                next if m/^\#/ || !m/\S/;
-                s/\s*\n$//;
-                m/^(\w[-:0-9A-Za-z]*)\=/ ||
-                    &error("bad line in substvars file $varlistfile at line $.");
-                $substvar{$1}= $';
-            }
-            close(SV);
-        } elsif ($! != ENOENT ) {
-            &error("unable to open substvars file $varlistfile: $!");
-        }
-    }
+    &parsesubstvars if ($dosubstvars);
     for $f (sort { $fieldimps{$b} <=> $fieldimps{$a} } keys %f) {
         $v= $f{$f};
         if ($dosubstvars) {
@@ -142,6 +131,7 @@ sub parsecontrolfile {
     $controlfile="./$controlfile" if $controlfile =~ m/^\s/;
 
     open(CDATA,"< $controlfile") || &error("cannot read control file $controlfile: $!");
+    binmode(CDATA);
     $indices= &parsecdata('C',1,"control file $controlfile");
     $indices >= 2 || &error("control file must have at least one binary package part");
 
@@ -150,10 +140,95 @@ sub parsecontrolfile {
             &error("per-package paragraph $i in control info file is ".
                    "missing Package line");
     }
+    defined($fi{"C Source"}) ||
+        &error("source paragraph in control info file is ".
+               "missing Source line");
+
+}
+
+my $substvarsparsed = 0;
+sub parsesubstvars {
+    if (length($varlistfile) && !$substvarsparsed) {
+        $varlistfile="./$varlistfile" if $varlistfile =~ m/\s/;
+        if (open(SV,"< $varlistfile")) {
+            binmode(SV);
+            while (<SV>) {
+                next if m/^\#/ || !m/\S/;
+                s/\s*\n$//;
+                m/^(\w[-:0-9A-Za-z]*)\=/ ||
+                    &error("bad line in substvars file $varlistfile at line $.");
+                $substvar{$1}= $';
+            }
+            close(SV);
+        } elsif ($! != ENOENT ) {
+            &error("unable to open substvars file $varlistfile: $!");
+        }
+        $substvarsparsed = 1;
+    }
+}
+
+sub parsedep {
+    my ($dep_line, $use_arch, $reduce_arch) = @_;
+    my @dep_list;
+    if (!$host_arch) {
+        $host_arch = `dpkg-architecture -qDEB_HOST_ARCH`;
+        chomp $host_arch;
+    }
+    foreach my $dep_and (split(/,\s*/m, $dep_line)) {
+        my @or_list = ();
+ALTERNATE:
+        foreach my $dep_or (split(/\s*\|\s*/m, $dep_and)) {
+            my ($package, $relation, $version);
+            $package = $1 if ($dep_or =~ s/^([a-zA-Z0-9][a-zA-Z0-9+._-]*)\s*//m);
+            ($relation, $version) = ($1, $2) if ($dep_or =~ s/^\((=|<=|>=|<<?|>>?)\s*([^)]+).*\)\s*//m);
+            my @arches = split(/\s+/m, $1) if ($use_arch && $dep_or =~ s/^\[([^]]+)\]\s*//m);
+            if ($reduce_arch && @arches) {
+
+                my $seen_arch='';
+                foreach my $arch (@arches) {
+                    $arch=lc($arch);
+                    if ($arch eq $host_arch) {
+                        $seen_arch=1;
+                        next;
+                    } elsif ($arch eq "!$host_arch") {
+                        next ALTERNATE;
+                    } elsif ($arch =~ /!/) {
+                        # This is equivilant to
+                        # having seen the current arch,
+                        # unless the current arch
+                        # is also listed..
+                        $seen_arch=1;
+                    }
+                }
+                if (! $seen_arch) {
+                    next;
+                }
+            }
+            return undef if (length($dep_or));
+	    push @or_list, [ $package, $relation, $version, \@arches ];
+        }
+        push @dep_list, \@or_list;
+    }
+    \@dep_list;
+}
+
+sub showdep {
+    my ($dep_list, $show_arch) = @_;
+    my @and_list;
+    foreach my $dep_and (@$dep_list) {
+        my @or_list = ();
+        foreach my $dep_or (@$dep_and) {
+            my ($package, $relation, $version, $arch_list) = @$dep_or; 
+            push @or_list, $package . ($relation && $version ? " ($relation $version)" : '') . ($show_arch && @$arch_list ? " [@$arch_list]" : '');
+        }
+        push @and_list, join(' | ', @or_list);
+    }
+    join(', ', @and_list);
 }
 
 sub parsechangelog {
     defined($c=open(CDATA,"-|")) || &syserr("fork for parse changelog");
+    binmode(CDATA);
     if (!$c) {
         @al=($parsechangelog);
         push(@al,"-F$changelogformat") if length($changelogformat);
@@ -186,11 +261,13 @@ sub parsecdata {
     while (<CDATA>) {
         s/\s*\n$//;
 	next if (m/^$/ and $paraborder);
+	next if (m/^#/);
 	$paraborder=0;
         if (m/^(\S+)\s*:\s*(.*)$/) {
             $cf=$1; $v=$2;
             $cf= &capit($cf);
             $fi{"$source$index $cf"}= $v;
+            $fi{"o:$source$index $cf"}= $1;
             if (lc $cf eq 'package') { $p2i{"$source $v"}= $index; }
         } elsif (m/^\s+\S/) {
             length($cf) || &syntax("continued value line not in field");
@@ -225,7 +302,9 @@ sub parsecdata {
 }
 
 sub unknown {
-    &warn("unknown information field $_ in input data in $_[0]");
+    my $field = $_;
+    my $key = (grep { /^o:C\d+ $field/ } keys %fi)[0];
+    &warn("unknown information field " . $f{"$key"} . " in input data in $_[0]");
 }
 
 sub syntax {
