@@ -1,25 +1,31 @@
 #!/usr/bin/perl --
 
-#use POSIX; &ENOENT;
-sub ENOENT { 2; }
-# Sorry about this, but the errno-part of POSIX.pm isn't in perl-*-base
-
 $version= '1.0.11'; # This line modified by Makefile
+$admindir= "/var/lib/dpkg"; # This line modified by Makefile
+$dpkglibdir= "../utils"; # This line modified by Makefile
+
+$enoent=`$dpkglibdir/enoent` || die "Cannot get ENOENT value from $dpkglibdir/enoent: $!";
+sub ENOENT { $enoent; }
+
 sub showversion {
-    print("Debian GNU/Linux dpkg-divert $version.\n") || &quit("failed to write version: $!");
+    print("Debian dpkg-divert $version.\n") || &quit("failed to write version: $!");
 }
 
 sub usage {
     &showversion;
-    print(STDOUT <<END)
-Copyright (C) 1995 Ian Jackson.  This is free software; see the GNU General
-Public Licence version 2 or later for copying conditions. There is NO warranty.
+    print STDERR <<EOF
+Copyright (C) 1995 Ian Jackson.
+Copyright (C) 2000,2001 Wichert Akkerman.
+
+This is free software; see the GNU General Public Licence version 2 or later
+for copying conditions. There is NO warranty.
 
 Usage:
 
  dpkg-divert [options] [--add] <file>               - add a diversion
  dpkg-divert [options] --remove <file>              - remove the diversion
  dpkg-divert [options] --list [<glob-pattern>]      - show file diversions
+ dpkg-divert [options] --truename <file>            - return the diverted file
 
 Options: 
     --package <package>        name of the package whose copy of <file>
@@ -36,11 +42,10 @@ Options:
 When adding, default is --local and --divert <original>.distrib.
 When removing, --package or --local and --divert must match if specified.
 Package preinst/postrm scripts should always specify --package and --divert.
-END
+EOF
         || &quit("failed to write usage: $!");
 }
 
-$admindir= '/var/lib/dpkg';
 $testmode= 0;
 $dorename= 0;
 $verbose= 1;
@@ -78,6 +83,9 @@ while (@ARGV) {
     } elsif (m/^--list$/) {
         &checkmanymodes;
         $mode= 'list';
+    } elsif (m/^--truename$/) {
+        &checkmanymodes;
+        $mode= 'truename';
     } elsif (m/^--divert$/) {
         @ARGV || &badusage("--divert needs a divert-to argument");
         $divertto= shift(@ARGV);
@@ -85,7 +93,7 @@ while (@ARGV) {
     } elsif (m/^--package$/) {
         @ARGV || &badusage("--package needs a package argument");
         $package= shift(@ARGV);
-        $divertto =~ m/\n/ && &badusage("package may not contain newlines");
+        $package =~ m/\n/ && &badusage("package may not contain newlines");
     } elsif (m/^--admindir$/) {
         @ARGV || &badusage("--admindir needs a directory argument");
         $admindir= shift(@ARGV);
@@ -109,9 +117,11 @@ close(O);
 if ($mode eq 'add') {
     @ARGV == 1 || &badusage("--add needs a single argument");
     $file= $ARGV[0];
+    $file =~ m#^/# || &badusage("filename \"$file\" is not absolute");
     $file =~ m/\n/ && &badusage("file may not contain newlines");
 	-d $file && &badusage("Cannot divert directories");
     $divertto= "$file.distrib" unless defined($divertto);
+    $divertto =~ m#^/# || &badusage("filename \"$divertto\" is not absolute");
     $package= ':' unless defined($package);
     for ($i=0; $i<=$#contest; $i++) {
         if ($contest[$i] eq $file || $altname[$i] eq $file ||
@@ -150,6 +160,7 @@ if ($mode eq 'add') {
                    ($i < $#altname ? @altname[$i+1..$#altname] : ()));
         @package= (($i > 0 ? @package[0..$i-1] : ()),
                    ($i < $#package ? @package[$i+1..$#package] : ()));
+	$dorename = 1;
         &checkrename($orgdivertto,$orgfile);
         &dorename($orgdivertto,$orgfile);
         &save;
@@ -165,7 +176,7 @@ if ($mode eq 'add') {
         s/\\\*/.*/g;
         push(@list,"^$_\$");
     }
-    $pat= join('$|^',@list);
+    $pat= join('|',@list);
     for ($i=0; $i<=$#contest; $i++) {
         next unless ($contest[$i] =~ m/$pat/o ||
                      $altname[$i] =~ m/$pat/o ||
@@ -173,6 +184,16 @@ if ($mode eq 'add') {
         print &infon($i),"\n";
     }
     exit(0);
+} elsif ($mode eq 'truename') {
+    @ARGV == 1 || &badusage("--truename needs a single argument");
+    $file= $ARGV[0];
+    for ($i=0; $i<=$#contest; $i++) {
+	next unless $file eq $contest[$i];
+	print $altname[$i], "\n";
+	exit(0);
+    }
+    print $file, "\n";
+    exit(0);
 } else {
     &quit("internal error - bad mode \`$mode'");
 }
@@ -199,13 +220,13 @@ sub checkrename {
     # (hopefully) wont overwrite anything. If it succeeds, we
     # assume a writable filesystem.
     foreach $file ($rsrc,$rdest) {
-	open (TMP, ">> ${file}.dpkg-devert.tmp") || $! == ENOENT ||
-		&quit("error checking \`$file': $!");
-	close TMP;
-	if ($1 == ENOENT) {
-		$dorename = 0;
-	} else {
+	if (open (TMP, ">> ${file}.dpkg-devert.tmp")) {
+		close TMP;
 		unlink ("${file}.dpkg-devert.tmp");
+	} elsif ($! == ENOENT) {
+		$dorename = !$dorename;
+	} else {
+		&quit("error checking \`$file': $!");
 	}
     }
     if (@ssrc && @sdest &&
