@@ -18,8 +18,8 @@ require 'controllib.pl';
 sub usageversion {
     print STDERR
 "Debian dpkg-gencontrol $version. 
-Copyright (C) 1996 Ian Jackson.
-Copyright (C) 2000 Wichert Akkerman.
+Copyright 1996 Ian Jackson.
+Copyright 2000,2002 Wichert Akkerman.
 This is free software; see the GNU General Public Licence version 2 or later
 for copying conditions.  There is NO warranty.
 
@@ -56,8 +56,8 @@ while (@ARGV) {
     $_=shift(@ARGV);
     if (m/^-p([-+0-9a-z.]+)$/) {
         $oppackage= $1;
-    } elsif (m/^-p([-+0-9a-zA-Z.]+)$/) {
-        &usageerr("Illegal package name \`$1'");
+    } elsif (m/^-p(.*)/) {
+        &error("Illegal package name \`$1'");
     } elsif (m/^-c/) {
         $controlfile= $';
     } elsif (m/^-l/) {
@@ -98,6 +98,7 @@ while (@ARGV) {
 
 &findarch;
 &parsechangelog;
+&parsesubstvars;
 &parsecontrolfile;
             
 if (length($oppackage)) {
@@ -112,6 +113,8 @@ if (length($oppackage)) {
 
 #print STDERR "myindex $myindex\n";
 
+my %pkg_dep_fields = map { $_ => 1 } @pkg_dep_fields;
+
 for $_ (keys %fi) {
     $v= $fi{$_};
     if (s/^C //) {
@@ -121,12 +124,12 @@ for $_ (keys %fi) {
         elsif (s/^X[CS]*B[CS]*-//i) { $f{$_}= $v; }
 	elsif (m/^X[CS]+-|^(Standards-Version|Uploaders)$|^Build-(Depends|Conflicts)(-Indep)?$/i) { }
 	elsif (m/^Section$|^Priority$/) { $spdefault{$_}= $v; }
-        else { &unknown('general section of control info file'); }
+        else { $_ = "C $_"; &unknown('general section of control info file'); }
     } elsif (s/^C$myindex //) {
 #print STDERR "P key >$_< value >$v<\n";
-        if (m/^(Package|Description|Essential|Pre-Depends|Depends)$/ ||
-            m/^(Recommends|Suggests|Enhances|Optional|Conflicts|Provides|Replaces)$/) {
+        if (m/^(Package|Description|Essential|Optional)$/) {
             $f{$_}= $v;
+        } elsif (exists($pkg_dep_fields{$_})) {
         } elsif (m/^Section$|^Priority$/) {
             $spvalue{$_}= $v;
         } elsif (m/^Architecture$/) {
@@ -144,7 +147,7 @@ for $_ (keys %fi) {
         } elsif (s/^X[CS]*B[CS]*-//i) {
             $f{$_}= $v;
         } elsif (!m/^X[CS]+-/i) {
-            &unknown("package's section of control info file");
+            $_ = "C$myindex $_"; &unknown("package's section of control info file");
         }
     } elsif (m/^C\d+ /) {
 #print STDERR "X key >$_< value not shown<\n";
@@ -159,8 +162,9 @@ for $_ (keys %fi) {
         } elsif (s/^X[CS]*B[CS]*-//i) {
             $f{$_}= $v;
         } elsif (!m/^X[CS]+-/i) {
-            &unknown("parsed version of changelog");
+            $_ = "L $_"; &unknown("parsed version of changelog");
         }
+    } elsif (m/o:/) {
     } else {
         &internerr("value from nowhere, with key >$_< and value >$v<");
     }
@@ -172,6 +176,26 @@ $origversion= $version; $origversion =~ s/-[^-]+$//;
 $substvar{"dpkg:UpstreamVersion"}=$origversion;
 $substvar{"dpkg:Version"}=$version;
 
+for $_ (keys %fi) {
+    $v= $fi{$_};
+    if (s/^C //) {
+    } elsif (s/^C$myindex //) {
+        if (m/^(Package|Description|Essential|Optional)$/) {
+        } elsif (exists($pkg_dep_fields{$_})) {
+            $f{$_}= showdep(parsedep(substvars($v)), 0);
+        } elsif (m/^Section$|^Priority$/) {
+        } elsif (m/^Architecture$/) {
+        } elsif (s/^X[CS]*B[CS]*-//i) {
+        } elsif (!m/^X[CS]+-/i) {
+        }
+    } elsif (m/^C\d+ /) {
+    } elsif (s/^L //) {
+    } elsif (m/o:/) {
+    } else {
+    }
+}
+
+
 for $f (qw(Section Priority)) {
     $spvalue{$f}= $spdefault{$f} unless length($spvalue{$f});
     $f{$f}= $spvalue{$f} if $spinclude{$f} && length($spvalue{$f});
@@ -211,15 +235,18 @@ if (length($substvar{'Installed-Size'})) {
 
 $fileslistfile="./$fileslistfile" if $fileslistfile =~ m/^\s/;
 open(Y,"> $fileslistfile.new") || &syserr("open new files list file");
+binmode(Y);
 chown(@fowner, "$fileslistfile.new") 
 		|| &syserr("chown new files list file");
 if (open(X,"< $fileslistfile")) {
+    binmode(X);
     while (<X>) {
         chomp;
         next if m/^([-+0-9a-z.]+)_[^_]+_(\w+)\.deb /
                 && ($1 eq $oppackage) && ($2 eq $arch || $2 eq 'all');
         print(Y "$_\n") || &syserr("copy old entry to new files list file");
     }
+    close(X) || &syserr("close old files list file");
 } elsif ($! != ENOENT) {
     &syserr("read old files list file");
 }
@@ -241,6 +268,7 @@ if (!$stdout) {
     $cf= "./$cf" if $cf =~ m/^\s/;
     open(STDOUT,"> $cf.new") ||
         &syserr("cannot open new output control file \`$cf.new'");
+    binmode(STDOUT);
 }
 &outputclose(1);
 if (!$stdout) {
