@@ -8,7 +8,7 @@ my %dirincluded;
 my %notfileobject;
 my $fn;
 
-$diff_ignore_default_regexp = '^.*~$|^\..*\.swp|DEADJOE|(?:/CVS|/RCS|/.deps)(?:$|/.*$)';
+$diff_ignore_default_regexp = '(?:^|/)\.#.*$|(?:^|/).*~$|(?:^|/)\..*\.swp|DEADJOE|\.cvsignore|(?:/(?:CVS|RCS|\.deps|\{arch\}|\.arch-ids|\.svn))(?:$|/.*$)';
 
 $sourcestyle = 'X';
 $dscformat = "1.0";
@@ -24,6 +24,15 @@ require 'controllib.pl';
 # Make sure patch doesn't get any funny ideas
 delete $ENV{'POSIXLY_CORRECT'};
 
+my @exit_handlers = ();
+sub exit_handler {
+	&$_ foreach ( reverse @exit_handlers );
+	exit(127);
+}
+$SIG{'INT'} = \&exit_handler;
+$SIG{'HUP'} = \&exit_handler;
+$SIG{'QUIT'} = \&exit_handler;
+
 sub usageversion {
     print STDERR
 "Debian dpkg-source $version.  Copyright (C) 1996
@@ -45,6 +54,7 @@ Build options:   -c<controlfile>     get control info from this file
                  -sa                 auto select orig source (-sA is default)
                  -i[<regexp>]        filter out files to ignore diffs of.
                                      Defaults to: '$diff_ignore_default_regexp'
+                 -I<filename>        filter out files when building tarballs.
                  -sk                 use packed orig source (unpack & keep)
                  -sp                 use packed orig source (unpack & remove)
                  -su                 use unpacked orig source (pack & keep)
@@ -96,6 +106,8 @@ while (@ARGV && $ARGV[0] =~ m/^-/) {
         $remove{$1}= 1;
     } elsif (m/^-i(.*)$/) {
         $diff_ignore_regexp = $1 ? $1 : $diff_ignore_default_regexp;
+    } elsif (m/^-I(.+)$/) {
+        push @tar_ignore, "--exclude=$1";
     } elsif (m/^-V(\w[-:0-9A-Za-z]*)[=:]/) {
         $substvar{$1}= $';
     } elsif (m/^-T/) {
@@ -141,11 +153,11 @@ if ($opmode eq 'build') {
     for $_ (keys %fi) {
         $v= $fi{$_};
         if (s/^C //) {
-            if (m/^Source$/) { &setsourcepackage; }
-            elsif (m/^(Standards-Version|Origin|Maintainer|Uploaders)$/) { $f{$_}= $v; }
+            if (m/^Source$/i) { &setsourcepackage; }
+            elsif (m/^(Standards-Version|Origin|Maintainer|Uploaders)$/i) { $f{$_}= $v; }
 	    elsif (m/^Build-(Depends|Conflicts)(-Indep)?$/i) { $f{$_}= $v; }
             elsif (s/^X[BC]*S[BC]*-//i) { $f{$_}= $v; }
-            elsif (m/^(Section|Priority|Files|Bugs)$/ || m/^X[BC]+-/i) { }
+            elsif (m/^(Section|Priority|Files|Bugs)$/i || m/^X[BC]+-/i) { }
             else { &unknown('general section of control info file'); }
         } elsif (s/^C(\d+) //) {
             $i=$1; $p=$fi{"C$i Package"};
@@ -169,14 +181,14 @@ if ($opmode eq 'build') {
                                    if grep($a eq $_, 'any','all');
                             push(@sourcearch,$a) unless $archadded{$a}++;
                         }
-		    }
+                }
                 }
                 $f{'Architecture'}= join(' ',@sourcearch);
             } elsif (s/^X[BC]*S[BC]*-//i) {
                 $f{$_}= $v;
-            } elsif (m/^(Package|Essential|Pre-Depends|Depends|Provides)$/ ||
-                     m/^(Recommends|Suggests|Optional|Conflicts|Replaces)$/ ||
-                     m/^(Description|Section|Priority)$/ ||
+            } elsif (m/^(Package|Essential|Pre-Depends|Depends|Provides)$/i ||
+                     m/^(Recommends|Suggests|Optional|Conflicts|Replaces)$/i ||
+                     m/^(Description|Section|Priority)$/i ||
                      m/^X[CS]+-/i) {
             } else {
                 &unknown("package's section of control info file");
@@ -188,11 +200,12 @@ if ($opmode eq 'build') {
                 $f{$_}= $v;
             } elsif (s/^X[BS]*C[BS]*-//i) {
                 $f{$_}= $v;
-            } elsif (m/^(Maintainer|Changes|Urgency|Distribution|Date|Closes)$/ ||
+            } elsif (m/^(Maintainer|Changes|Urgency|Distribution|Date|Closes)$/i ||
                      m/^X[BS]+-/i) {
             } else {
                 &unknown("parsed version of changelog");
             }
+        } elsif (m/^o:.*/) {
         } else {
             &internerr("value from nowhere, with key >$_< and value >$v<");
         }
@@ -303,7 +316,7 @@ if ($opmode eq 'build') {
             chdir($tardirbase) || &syserr("chdir to above (orig) source $tardirbase");
             open(STDOUT,">&GZIP") || &syserr("reopen gzip for tar");
             # FIXME: put `--' argument back when tar is fixed
-            exec('tar','-cf','-',$tardirname); &syserr("exec tar");
+            exec('tar',@tar_ignore,'-cf','-',$tardirname); &syserr("exec tar");
         }
         close(GZIP);
         &reapgzip;
@@ -327,7 +340,9 @@ if ($opmode eq 'build') {
             $sourcestyle =~ m/[KP]/ ||
                 &error("orig dir \`$origdir' already exists, not overwriting,".
                        " giving up; use -sA, -sK or -sP to override");
+	    push @exit_handlers, sub { erasedir($origdir) };
             erasedir($origdir);
+	    pop @exit_handlers;
         } elsif ($! != ENOENT) {
             &syserr("unable to check for existence of orig dir \`$origdir'");
         }
@@ -338,13 +353,14 @@ if ($opmode eq 'build') {
 #        checktarsane($origtargz,$expectprefix);
         mkdir("$origtargz.tmp-nest",0755) ||
             &syserr("unable to create \`$origtargz.tmp-nest'");
+	push @exit_handlers, sub { erasedir("$origtargz.tmp-nest") };
         extracttar($origtargz,"$origtargz.tmp-nest",$expectprefix);
         rename("$origtargz.tmp-nest/$expectprefix",$expectprefix) ||
             &syserr("unable to rename \`$origtargz.tmp-nest/$expectprefix' to ".
                     "\`$expectprefix'");
         rmdir("$origtargz.tmp-nest") ||
             &syserr("unable to remove \`$origtargz.tmp-nest'");
-
+	    pop @exit_handlers;
     }
         
     if ($sourcestyle =~ m/[kpursKPUR]/) {
@@ -389,6 +405,7 @@ if ($opmode eq 'build') {
                 if (!$c3) {
 		    $ENV{'LC_ALL'}= 'C';
 		    $ENV{'LANG'}= 'C';
+		    $ENV{'TZ'}= 'UTC0';
                     exec('diff','-u',
                          '-L',"$basedirname.orig/$fn",
                          '-L',"$basedirname/$fn",
@@ -503,7 +520,7 @@ if ($opmode eq 'build') {
     if (defined $fi{'S Format'}) {
 	if (not handleformat($fi{'S Format'})) {
 	    &error("Unsupported format of .dsc file ($dscformat)");
-        }
+	}
         $dscformat=$fi{'S Format'};
     }
 
@@ -513,11 +530,11 @@ if ($opmode eq 'build') {
     $sourcepackage= $fi{'S Source'};
     $sourcepackage =~ m/[^-+.0-9a-z]/ &&
         &error("source package name contains illegal character \`$&'");
-    $sourcepackage =~ m/^[0-9a-z]./ ||
-        &error("source package name is too short or starts with non-alphanum");
+    $sourcepackage =~ m/^[0-9a-z]/ ||
+        &error("source package name starts with non-alphanum");
 
     $version= $fi{'S Version'};
-    $version =~ m/[^-+:.0-9a-zA-Z]/ &&
+    $version =~ m/[^-+:.0-9a-zA-Z~]/ &&
         &error("version number contains illegal character \`$&'");
     $version =~ s/^\d+://;
     if ($version =~ m/-([^-]+)$/) {
@@ -529,7 +546,7 @@ if ($opmode eq 'build') {
     $files= $fi{'S Files'};
     for $file (split(/\n /,$files)) {
         next if $file eq '';
-        $file =~ m/^([0-9a-f]{32})[ \t]+(\d+)[ \t]+([0-9a-zA-Z][-+:.,=0-9a-zA-Z_]+)$/
+        $file =~ m/^([0-9a-f]{32})[ \t]+(\d+)[ \t]+([0-9a-zA-Z][-+:.,=0-9a-zA-Z_~]+)$/
             || &error("Files field contains bad line \`$file'");
         ($md5sum{$3},$size{$3},$file) = ($1,$2,$3);
         &setfile(\$tarfile) if $file =~ m/\.tar\.gz$/;
@@ -989,7 +1006,7 @@ sub extracttar {
 }
 
 sub cpiostderr {
-    open(STDERR,"| egrep -v '^[0-9]+ blocks\$' >&2") ||
+    open(STDERR,"| grep -E -v '^[0-9]+ blocks\$' >&2") ||
         &syserr("reopen stderr for tar to grep out blocks message");
 }
 
@@ -1071,7 +1088,7 @@ sub addfile {
     $size= (stat _)[7];
     my $md5sum= `md5sum <$filename`;
     $? && &subprocerr("md5sum $filename");
-    $md5sum =~ s/^([0-9a-f]{32})\n$/$1/ || &failure("md5sum gave bogus output \`$_'");
+    $md5sum =~ s/^([0-9a-f]{32})\s*-?\s*\n$/$1/ || &failure("md5sum gave bogus output \`$_'");
     $f{'Files'}.= "\n $md5sum $size $filename";
 }
 
